%  This analyzes results from the CEE_ACEL model.
%
%  Input:
%  The observed data are Y(:,t), t=1,...,T.
%  Observer equation: Y(:,t)=H*ksi(:,t)+w(:,t), w iid, Ew*w'=RR.
%  State evolution equation: ksi(:,t+1)=F*ksi(:,t)+nu(:,t+1), nu iid, Enu*nu'=QQ.
%  P unconditional variance of ksi (This is P1|0 in Hamilton's notation.)
%
%  Output:
%  Yhat(:,t)=E[H*ksi(:,t)|information(t1)], for t=t1,...,t1+L,
%  where t1>=1, information(t1)={Y(:,1),...,Y(:,t1)}.
%  VAR(:,:,t) = variance of Yhat(:,t) forecast given information(t1), for
%  t=t1,...,t1+L.
%
%Q(11,11)=0;
RR=H;
F=T;
QQ=R*Q*R';
P=Pstar;
H=Z;
mm=mean(Y')';
YY=Y-repmat(mm,1,size(Y,2));
[xsmooth, Vsmooth, VVsmooth, loglik] = kalman_smoother(YY, F, H, QQ, RR, zeros(size(H,2),1), P);


%order of variables in smoothed state (xsmooth) is given above

%order of observable variables in YY
%dly linfl dlc dli RXU dlgd dlpi dlh 

ZZ=strvcat('dlc','linfl','dlc','dli','RXU','dlgd','dlpi','dlh');

for ii = 1:size(ZZ,1)
    ZII(ii)=strmatch(ZZ(ii,:),lgy_);
end
ZIIi=dr_.inv_order_var(ZII);

%dbstop in ceeoutput at 83

for ii = 1:size(Q,1);
    QQQ=zeros(size(Q,1));
    QQQ(ii,ii)=Q(ii,ii);
    V = lyapunov_symm(F,R*QQQ*transpose(R));
    bb(:,ii)=diag(V);
end

tvariance=sum(bb')';
for ii = 1:length(ZIIi)
    a=[];
    for jj = 1:size(Q,1)
        a=[a ,' due to shock ',lgx_(jj,:),' = %6.5f\n'];
    end
    a=['\n variance decomposition in ',ZZ(ii,:),'\n', a];
    fprintf(a,100*bb(ZIIi(ii),:)/tvariance(ZIIi(ii)))
end

return

start=1;
figure('Name','Raw data and smoothed counterpart')
subplot(231)
plot(time,400*xsmooth(ZIIi(1),:),time,400*YY(5,:),'*-')
sss=['std dev raw data (APR) = ',num2str(400*std(YY(5,:)))];
ssx=['std dev model (APR) = ',num2str(400*sqrt(Pstar(ZIIi(1),ZIIi(1))))];
text(1985,-.006,sss);
text(1985,-.008,ssx);
title('consumption growth')
axis tight
legend('smoothed','actual')
subplot(232)
plot(time,400*xsmooth(ZIIi(2),:),time,400*YY(4,:),'*-')
sss=['std dev raw data (APR) = ',num2str(400*std(YY(4,:)))];
ssx=['std dev model (APR) = ',num2str(400*sqrt(Pstar(ZIIi(2),ZIIi(2))))];
text(1985,.05,sss);
text(1985,.04,ssx);
axis tight
title('investment growth')
subplot(233)
plot(time,400*xsmooth(ZIIi(3),:),time,400*YY(1,:),'*-')
sss=['std dev raw data (APR) = ',num2str(400*std(YY(1,:)))];
ssx=['std dev model (APR) = ',num2str(400*sqrt(Pstar(ZIIi(3),ZIIi(3))))];
text(1985,-.01,sss);
text(1985,-.0125,ssx);
axis tight
title('output growth')
subplot(234)
plot(time,xsmooth(ZIIi(4),:),time,YY(3,:),'*-')
sss=['std dev raw data (percent) = ',num2str(100*std(YY(3,:)))];
ssx=['std dev model (percent) = ',num2str(100*sqrt(Pstar(ZIIi(4),ZIIi(4))))];
text(1985,.06,sss);
text(1985,.04,ssx);
axis tight
title('log hours')
subplot(235)
plot(time,400*xsmooth(ZIIi(5),:),time,400*YY(2,:),'*-')
sss=['std dev raw data (APR) = ',num2str(400*std(YY(2,:)))];
ssx=['std dev model (APR) = ',num2str(400*sqrt(Pstar(ZIIi(5),ZIIi(5))))];
text(1985,-.003,sss);
text(1985,-.004,ssx);
axis tight
title('inflation')
subplot(236)
plot(time,400*xsmooth(ZIIi(6),:),time,400*YY(6,:),'*-')
sss=['std dev raw data (APR) = ',num2str(400*std(YY(6,:)))];
ssx=['std dev model (APR) = ',num2str(400*sqrt(Pstar(ZIIi(6),ZIIi(6))))];
text(1985,-.005,sss);
text(1985,-.009,ssx);
axis tight
title('interest rate')
hout=suptitle('demeaned data versus smoothed version');

%compute the innovations in the state, called nu above
for ii = 2:size(YY,2)
    nu(:,ii)=xsmooth(:,ii)-F*xsmooth(:,ii-1);
end

AA=strvcat('epsilU','lambdafU','muupU','serialmps','zetacU','zetaiU','zetaU','muzstarU','gU');
Aa=strvcat('epsilx','lambdafx','muupx','mps','zetacx','zetaix','zetax','muzstarx','gx');

ix=1;
II=[];
for ii = 1:size(AA,1)
    for jj = 1:size(lgy_,1)
        if strcmpi(deblank(AA(ii,:)),deblank(lgy_(jj,:))) == 1
            II(ix,1)=jj;
            for ll = 1:size(lgx_,1)
                if strcmpi(deblank(Aa(ii,:)),deblank(lgx_(ll,:))) == 1
                    II(ix,2)=ll;
                end
            end
            ix=ix+1;
        end
    end
end

%II(:,1) place of shock in lgy_
%II(:,2) place of corresponding shock innovation in lgx_
IIi=[];
if isempty(II)~=1
    IIi(:,1)=dr_.inv_order_var(II(:,1));
else
    IIi=[];
end

%CC=strvcat('signalstate1','signalstate10','signalstate11','signalstate12','signalstate2','signalstate3','signalstate4','signalstate5','signalstate6','signalstate7','signalstate8','signalstate9');
%Cc=strvcat('sig1','sig10','sig11','sig12','sig2','sig3','sig4','sig5','sig6','sig7','sig8','sig9');
CC=strvcat('signalstate1','signalstate2','signalstate3','signalstate4','signalstate5','signalstate6','signalstate7','signalstate8');
Cc=strvcat('sig1','sig2','sig3','sig4','sig5','sig6','sig7','sig8');
%CcII(:,1) identifies signals that are estimated
%CcII(:,2) indicates the location in lgx_ of the estimated signal
ix=1;
CcII=[];
for ii = 1:size(Cc,1)
    for jj = 1:size(lgx_)
        if strcmpi(deblank(Cc(ii,:)),deblank(lgx_(jj,:))) == 1
            CcII(ix,1)=ii;
            CcII(ix,2)=jj;
            ix=ix+1;
        end
    end
end
if isempty(CcII) ~= 1
    CCcc=CC(CcII(:,1),:);
    ix=1;
    CII=[];
    %determine the index in lgy_ of shocks affected by the signal shocks that
    %are included in estimation
    for ii = 1:size(CCcc,1)
        for jj = 1:length(lgy_)
            if strcmpi(deblank(CCcc(ii,:)),deblank(lgy_(jj,:))) == 1
                CII(ix)=jj;
                ix=ix+1;
            end
        end
    end

    if isempty(CII)~=1
        CIIi=dr_.inv_order_var(CII);
    else
        CIIi=[];
    end
    ip=0;
else
    CIIi=[];
    ip=1;
end
%kk is the index of the state variables (in reordered form) that are hit by
%the innovations included in estimation
kk=[IIi' CIIi];
kk=sort(kk);
shk=xsmooth(kk,:);

if rank(R(kk,:)) ~= length(kk)
    error('fatal (ceeoutput) R matrix singular')
end
inn=inv(R(kk,:))*nu(kk,:);
ksi(:,1)=xsmooth(:,1);
yy(:,1)=H*ksi(:,1);
yyhat(:,1)=H*xsmooth(:,1);
for ii = 2:size(inn,2);
    ksi(:,ii)=F*ksi(:,ii-1)+R*inn(:,ii);
    yy(:,ii)=H*ksi(:,ii);
    yyhat(:,ii)=H*xsmooth(:,ii);
end

if max(max(abs(yy-yyhat))) > 3.2e-6
    error('fatal (ceeoutput) something wrong with the computation of the innovations')
end

%dly piU lhU dli dlc RXU
ivar=1;
APR=1;
ceedecomposition(ivar,APR,inn,yyhat,H,F,R,time,kk,order_var,lgy_,lgx_,xsmooth);
ivar=2;
ceedecomposition(ivar,APR,inn,yyhat,H,F,R,time,kk,order_var,lgy_,lgx_,xsmooth);
ivar=3;
APR=0;
ceedecomposition(ivar,APR,inn,yyhat,H,F,R,time,kk,order_var,lgy_,lgx_,xsmooth);
ivar=4;
APR=1;
ceedecomposition(ivar,APR,inn,yyhat,H,F,R,time,kk,order_var,lgy_,lgx_,xsmooth);
ivar=5;
ceedecomposition(ivar,APR,inn,yyhat,H,F,R,time,kk,order_var,lgy_,lgx_,xsmooth);
ivar=6;
Apr=0;
ceedecomposition(ivar,APR,inn,yyhat,H,F,R,time,kk,order_var,lgy_,lgx_,xsmooth);

if ip == 1
    return
end



%figure('Name','Smoothed innovations to shocks')
tt=2:length(time);
ss=std(inn');
index=[II(:,1) II(:,2)
    CII' CcII(:,2) ];
ia=4;
ib=4;

%order of innovation names and variances:
bb=lgx_(estim_params_.var_exo(:,1),:);

ix=1;
IJ=[];
for ii = 1:size(bb,1)
    for jj = 1:size(bb,1)
        if strcmpi(deblank(bb(ii,:)),deblank(lgx_(index(jj,2),:))) == 1
            IJ(ix)=jj;
            ix=ix+1;
        end
    end
end

% if ia*ib >= estim_params_.nvx
%     for ii = 1:estim_params_.nvx
%         subplot(ia,ib,ii)
%         plot(time(tt),inn(IJ(ii),tt))
%         ssy=['sample std dev (inn) = ',num2str(ss(IJ(ii)))];
%         ssx=['model std dev (inn) = ',num2str(xparam1(ii))];
%         text(1995,-.00,ssy);
%         text(1995,-.0,ssx);
%         axis tight
%         gg=deblank(lgx_(estim_params_.var_exo(ii,1),:));
%         title(gg)
%     end
%     
% end
%this is where the place of the signal 8 is
place=11;
for ttt=9:length(time)
    eps85(ttt)=inn(place,ttt-8)+inn(place-1,ttt-7)+inn(place-2,ttt-6)+inn(place-3,ttt-5);

    eps41(ttt)=inn(place-4,ttt-4)+inn(place-5,ttt-3)+inn(place-6,ttt-2)+inn(place-7,ttt-1);

    eps0(ttt)=inn(1,ttt);
    epst(ttt)=eps85(ttt)+eps41(ttt)+eps0(ttt);
end
save innov eps85 eps41 eps0 epst inn time IJ



%&&&&&&&&&&&&&&&&&&&&&&
%the ordering and labeling here are 'probably' ok.
figure('Name','Smoothed Shocks')
tt=2:length(time);
ss=std(shk');
for ii = 1:length(kk)
    subplot(4,4,ii)
    plot(time(tt),shk(ii,tt))
    ssy=['sample std dev = ',num2str(ss(1))];
    text(1995,-.00,ssy);
    axis tight
    gg=deblank(lgy_(dr_.order_var(kk(ii)),:));
    title(gg)
end
hout=suptitle('smoothed shocks');

disp(' correlations among estimated innovations of the following shocks')
lgy_(dr_.order_var(kk),:)
format bank
corr(inn(:,[2:end])')
format long